User's Guide
------------

This part of the documentation will show you how to get started in using
Flask-RESTful with Flask.

.. toctree::
   :maxdepth: 2

   installation
   quickstart
   reqparse 
   fields
   extending
   intermediate-usage

API Reference
-------------

If you are looking for information on a specific function, class or
method, this part of the documentation is for you.

.. toctree::
   :maxdepth: 2

   api

Additional Notes
----------------

See Flask's license_ for legal information governing this project.

.. toctree::
   :maxdepth: 2

   testing

.. _license: http://flask.pocoo.org/docs/license/

